"""测试服务存储。"""

import os
import tempfile

import pytest

from autostartx.config import ConfigManager
from autostartx.models import ServiceStatus
from autostartx.storage import ServiceStorage


@pytest.fixture
def storage():
    """创建测试用的存储实例。"""
    with tempfile.TemporaryDirectory() as temp_dir:
        config_path = os.path.join(temp_dir, "test_config.toml")
        config_manager = ConfigManager(config_path)
        # 确保使用独立的数据目录
        config_manager.config.data_dir = os.path.join(temp_dir, "data")
        config_manager.config.log_dir = os.path.join(temp_dir, "logs")
        yield ServiceStorage(config_manager)


def test_add_service(storage):
    """测试添加服务。"""
    service = storage.add_service(name="test-service", command="echo hello", auto_restart=True)

    assert service.name == "test-service"
    assert service.command == "echo hello"
    assert service.auto_restart is True
    assert len(service.id) == 8  # UUID前8位
    assert service.status == ServiceStatus.STOPPED


def test_add_duplicate_service(storage):
    """测试添加重复名称的服务。"""
    storage.add_service(name="test-service", command="echo hello")

    with pytest.raises(ValueError, match="Service name 'test-service' already exists"):
        storage.add_service(name="test-service", command="echo world")


def test_get_service(storage):
    """测试获取服务。"""
    # 添加服务
    original_service = storage.add_service(name="test-service", command="echo hello")

    # 按ID获取
    service = storage.get_service(original_service.id)
    assert service is not None
    assert service.id == original_service.id
    assert service.name == "test-service"

    # 获取不存在的服务
    assert storage.get_service("nonexistent") is None


def test_get_service_by_name(storage):
    """测试按名称获取服务。"""
    # 添加服务
    original_service = storage.add_service(name="test-service", command="echo hello")

    # 按名称获取
    service = storage.get_service_by_name("test-service")
    assert service is not None
    assert service.name == "test-service"
    assert service.id == original_service.id

    # 获取不存在的服务
    assert storage.get_service_by_name("nonexistent") is None


def test_find_service(storage):
    """测试查找服务（ID或名称）。"""
    # 添加服务
    original_service = storage.add_service(name="test-service", command="echo hello")

    # 按ID查找
    service = storage.find_service(original_service.id)
    assert service is not None
    assert service.id == original_service.id

    # 按名称查找
    service = storage.find_service("test-service")
    assert service is not None
    assert service.name == "test-service"

    # 查找不存在的服务
    assert storage.find_service("nonexistent") is None


def test_get_all_services(storage):
    """测试获取所有服务。"""
    # 初始应该为空
    assert len(storage.get_all_services()) == 0

    # 添加服务
    storage.add_service(name="service-1", command="echo 1")
    storage.add_service(name="service-2", command="echo 2")

    services = storage.get_all_services()
    assert len(services) == 2

    service_names = [s.name for s in services]
    assert "service-1" in service_names
    assert "service-2" in service_names


def test_update_service(storage):
    """测试更新服务。"""
    # 添加服务
    service = storage.add_service(name="test-service", command="echo hello")
    original_updated_at = service.updated_at

    # 修改服务
    service.update_status(ServiceStatus.RUNNING)
    service.pid = 1234

    # 更新存储
    storage.update_service(service)

    # 重新获取验证
    updated_service = storage.get_service(service.id)
    assert updated_service.status == ServiceStatus.RUNNING
    assert updated_service.pid == 1234
    assert updated_service.updated_at > original_updated_at


def test_remove_service(storage):
    """测试删除服务。"""
    # 添加服务
    service = storage.add_service(name="test-service", command="echo hello")
    service_id = service.id

    # 确认服务存在
    assert storage.get_service(service_id) is not None

    # 删除服务
    result = storage.remove_service(service_id)
    assert result is True

    # 确认服务已删除
    assert storage.get_service(service_id) is None

    # 删除不存在的服务
    result = storage.remove_service("nonexistent")
    assert result is False


def test_get_services_by_status(storage):
    """测试按状态获取服务。"""
    # 添加服务
    service1 = storage.add_service(name="service-1", command="echo 1")
    service2 = storage.add_service(name="service-2", command="echo 2")

    # 修改状态
    service1.update_status(ServiceStatus.RUNNING)
    storage.update_service(service1)

    # 测试按状态获取
    running_services = storage.get_services_by_status(ServiceStatus.RUNNING)
    stopped_services = storage.get_services_by_status(ServiceStatus.STOPPED)

    assert len(running_services) == 1
    assert len(stopped_services) == 1
    assert running_services[0].id == service1.id
    assert stopped_services[0].id == service2.id


def test_persistence(storage):
    """测试数据持久化。"""
    # 添加服务
    service = storage.add_service(name="test-service", command="echo hello")
    service_id = service.id

    # 创建新的存储实例（模拟重启）
    new_storage = ServiceStorage(storage.config_manager)

    # 验证数据已持久化
    loaded_service = new_storage.get_service(service_id)
    assert loaded_service is not None
    assert loaded_service.name == "test-service"
    assert loaded_service.command == "echo hello"



"""Logging management system."""

import gzip
import logging
import os
import time
from pathlib import Path
from typing import Any, Dict, List, Optional

from .config import ConfigManager


class LogManager:
    """Log manager."""

    def __init__(self, config_manager: ConfigManager):
        self.config_manager = config_manager
        self._setup_logging()

    def _setup_logging(self) -> None:
        """Setup application logging."""
        # Create log directory
        log_dir = Path(self.config_manager.config.log_dir)
        log_dir.mkdir(parents=True, exist_ok=True)

        # Configure main log
        log_file = log_dir / "autostartx.log"

        # Set log format
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

        # File handler
        file_handler = logging.FileHandler(log_file, encoding="utf-8")
        file_handler.setFormatter(formatter)
        file_handler.setLevel(getattr(logging, self.config_manager.config.log_level))

        # Console handler
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(formatter)
        console_handler.setLevel(logging.WARNING)  # Only show warnings and errors on console

        # Configure root logger
        root_logger = logging.getLogger("autostartx")
        root_logger.setLevel(getattr(logging, self.config_manager.config.log_level))
        root_logger.addHandler(file_handler)
        root_logger.addHandler(console_handler)

        # Avoid duplicate handlers
        root_logger.propagate = False

    def get_service_log_path(self, service_id: str) -> str:
        """Get service log path."""
        return self.config_manager.get_service_log_path(service_id)

    def read_service_logs(
        self, service_id: str, lines: int = 100, since: Optional[float] = None
    ) -> List[str]:
        """Read service logs."""
        log_path = self.get_service_log_path(service_id)

        if not os.path.exists(log_path):
            return []

        try:
            with open(log_path, encoding="utf-8") as f:
                all_lines = f.readlines()

            # If time filter is specified
            if since:
                filtered_lines = []
                for line in all_lines:
                    # Try to parse timestamp (simplified version)
                    if self._line_is_after_time(line, since):
                        filtered_lines.append(line)
                all_lines = filtered_lines

            # Return last N lines
            return all_lines[-lines:] if lines > 0 else all_lines

        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to read service logs: {e}")
            return []

    def clear_service_logs(self, service_id: str) -> bool:
        """Clear service logs."""
        log_path = self.get_service_log_path(service_id)

        try:
            with open(log_path, "w", encoding="utf-8") as f:
                f.write("")
            logging.getLogger("autostartx").info(f"Cleared logs for service {service_id}")
            return True
        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to clear service logs: {e}")
            return False

    def rotate_service_logs(self, service_id: str) -> bool:
        """Rotate service logs."""
        log_path = self.get_service_log_path(service_id)

        if not os.path.exists(log_path):
            return True

        try:
            # Check file size
            file_size = os.path.getsize(log_path)
            max_size = self._parse_size(self.config_manager.config.max_log_size)

            if file_size < max_size:
                return True

            # Perform rotation
            timestamp = int(time.time())
            rotated_path = f"{log_path}.{timestamp}"

            # Rename current log file
            os.rename(log_path, rotated_path)

            # Compress old log
            self._compress_log_file(rotated_path)

            # Create new empty log file
            with open(log_path, "w", encoding="utf-8") as f:
                f.write(f"=== Log rotation: {time.strftime('%Y-%m-%d %H:%M:%S')} ===\n")

            logging.getLogger("autostartx").info(f"Rotated logs for service {service_id}")
            return True

        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to rotate logs: {e}")
            return False

    def cleanup_old_logs(self) -> None:
        """Clean up expired logs."""
        log_dir = Path(self.config_manager.config.log_dir)
        if not log_dir.exists():
            return

        retention_days = self.config_manager.config.log_retention_days
        cutoff_time = time.time() - (retention_days * 24 * 3600)

        try:
            for log_file in log_dir.glob("*.log.*"):
                if log_file.stat().st_mtime < cutoff_time:
                    log_file.unlink()
                    logging.getLogger("autostartx").info(f"Deleted expired log: {log_file}")

            # Clean up compressed logs
            for gz_file in log_dir.glob("*.gz"):
                if gz_file.stat().st_mtime < cutoff_time:
                    gz_file.unlink()
                    logging.getLogger("autostartx").info(
                        f"Deleted expired compressed log: {gz_file}"
                    )

        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to clean up expired logs: {e}")

    def get_log_stats(self, service_id: str) -> Dict[str, Any]:
        """Get log statistics."""
        log_path = self.get_service_log_path(service_id)

        if not os.path.exists(log_path):
            return {
                "exists": False,
                "size": 0,
                "lines": 0,
                "last_modified": None,
            }

        try:
            stat = os.stat(log_path)

            # Count lines
            with open(log_path, encoding="utf-8") as f:
                lines = sum(1 for _ in f)

            return {
                "exists": True,
                "size": stat.st_size,
                "size_mb": stat.st_size / 1024 / 1024,
                "lines": lines,
                "last_modified": stat.st_mtime,
                "last_modified_str": time.strftime(
                    "%Y-%m-%d %H:%M:%S", time.localtime(stat.st_mtime)
                ),
            }

        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to get log statistics: {e}")
            return {"exists": False, "error": str(e)}

    def _parse_size(self, size_str: str) -> int:
        """Parse size string to bytes."""
        size_str = size_str.upper()

        if size_str.endswith("KB"):
            return int(size_str[:-2]) * 1024
        elif size_str.endswith("MB"):
            return int(size_str[:-2]) * 1024 * 1024
        elif size_str.endswith("GB"):
            return int(size_str[:-2]) * 1024 * 1024 * 1024
        else:
            # Assume bytes
            return int(size_str)

    def _compress_log_file(self, file_path: str) -> None:
        """Compress log file."""
        try:
            compressed_path = f"{file_path}.gz"

            with open(file_path, "rb") as f_in:
                with gzip.open(compressed_path, "wb") as f_out:
                    f_out.writelines(f_in)

            # Delete original file
            os.remove(file_path)

        except Exception as e:
            logging.getLogger("autostartx").error(f"Failed to compress log file: {e}")

    def _line_is_after_time(self, line: str, since_time: float) -> bool:
        """Check if log line is after specified time."""
        # This is a simplified version, should parse log timestamps in reality
        # Here assumes log lines don't contain timestamps, or use file modification time
        return True


class ServiceLogRotator:
    """Service log rotator."""

    def __init__(self, log_manager: LogManager):
        self.log_manager = log_manager

    def rotate_if_needed(self, service_id: str) -> bool:
        """Rotate logs if needed."""
        stats = self.log_manager.get_log_stats(service_id)

        if not stats.get("exists"):
            return True

        max_size = self.log_manager._parse_size(self.log_manager.config_manager.config.max_log_size)

        if stats["size"] >= max_size:
            return self.log_manager.rotate_service_logs(service_id)

        return True

    def rotate_all_services(self, service_ids: List[str]) -> Dict[str, bool]:
        """Rotate logs for all services."""
        results = {}

        for service_id in service_ids:
            results[service_id] = self.rotate_if_needed(service_id)

        return results



"""Service manager - core class integrating all functionality."""

import time
from typing import Any, Dict, List, Optional

from .config import ConfigManager
from .models import ServiceInfo, ServiceStatus
from .process_manager import ProcessManager
from .storage import ServiceStorage


class ServiceManager:
    """Service manager."""

    def __init__(self, config_path: Optional[str] = None):
        self.config_manager = ConfigManager(config_path)
        self.storage = ServiceStorage(self.config_manager)
        self.process_manager = ProcessManager(self.config_manager)

    def add_service(
        self,
        name: str,
        command: str,
        auto_restart: bool = True,
        working_dir: str = "",
        env_vars: Optional[Dict[str, str]] = None,
    ) -> ServiceInfo:
        """Add new service."""
        service = self.storage.add_service(
            name=name,
            command=command,
            auto_restart=auto_restart,
            working_dir=working_dir,
            env_vars=env_vars,
        )
        return service

    def start_service(self, service_id_or_name: str) -> bool:
        """Start service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        success = self.process_manager.start_service(service)
        if success:
            # Mark service as auto-startable when manually started
            service.auto_start = True
            self.storage.update_service(service)
        return success

    def stop_service(self, service_id_or_name: str, force: bool = False) -> bool:
        """Stop service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        success = self.process_manager.stop_service(service, force)
        if success:
            self.storage.update_service(service)
        return success

    def restart_service(self, service_id_or_name: str, force: bool = False) -> bool:
        """Restart service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        success = self.process_manager.restart_service(service, force)
        if success:
            service.increment_restart_count()
            self.storage.update_service(service)
        return success

    def pause_service(self, service_id_or_name: str) -> bool:
        """Pause service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        success = self.process_manager.pause_service(service)
        if success:
            self.storage.update_service(service)
        return success

    def resume_service(self, service_id_or_name: str) -> bool:
        """Resume service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        success = self.process_manager.resume_service(service)
        if success:
            self.storage.update_service(service)
        return success

    def remove_service(self, service_id_or_name: str, force: bool = False) -> bool:
        """Remove service."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        # If service is running, stop it first
        if service.status == ServiceStatus.RUNNING:
            if not force:
                # Return False to let CLI handle the user-friendly message
                return False
            # Force stop the service
            if not self.stop_service(service.id, force=True):
                return False

        return self.storage.remove_service(service.id)

    def get_service(self, service_id_or_name: str) -> Optional[ServiceInfo]:
        """Get service information."""
        return self.storage.find_service(service_id_or_name)

    def list_services(self) -> List[ServiceInfo]:
        """Get all services list."""
        services = self.storage.get_all_services()

        # Update service status
        for service in services:
            self._update_service_status(service)

        return services

    def get_service_status(self, service_id_or_name: str) -> Optional[Dict[str, Any]]:
        """Get detailed service status."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return None

        # Update status
        self._update_service_status(service)

        # Get process information
        process_info = self.process_manager.get_process_info(service)

        status_info = {
            "service": service,
            "process": process_info,
            "uptime": self._calculate_uptime(service, process_info),
        }

        return status_info

    def get_service_logs(self, service_id_or_name: str, lines: int = 100) -> Optional[List[str]]:
        """Get service logs."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return None

        log_path = self.config_manager.get_service_log_path(service.id)

        try:
            with open(log_path, encoding="utf-8") as f:
                all_lines = f.readlines()
                return all_lines[-lines:] if lines > 0 else all_lines
        except FileNotFoundError:
            return []
        except Exception:
            return None

    def clear_service_logs(self, service_id_or_name: str) -> bool:
        """Clear service logs."""
        service = self.storage.find_service(service_id_or_name)
        if not service:
            return False

        log_path = self.config_manager.get_service_log_path(service.id)

        try:
            with open(log_path, "w", encoding="utf-8") as f:
                f.write("")
            return True
        except Exception:
            return False

    def _update_service_status(self, service: ServiceInfo) -> None:
        """Update service status."""
        if service.pid:
            if self.process_manager.is_process_running(service.pid):
                # Process exists and status is not running, update status
                if (
                    service.status != ServiceStatus.RUNNING
                    and service.status != ServiceStatus.PAUSED
                ):
                    service.update_status(ServiceStatus.RUNNING)
                    self.storage.update_service(service)
            else:
                # Process doesn't exist, update status
                service.pid = None
                service.update_status(ServiceStatus.STOPPED)
                self.storage.update_service(service)
        else:
            # No PID and status is not stopped, update status
            if service.status != ServiceStatus.STOPPED:
                service.update_status(ServiceStatus.STOPPED)
                self.storage.update_service(service)

    def _calculate_uptime(
        self, service: ServiceInfo, process_info: Optional[Dict[str, Any]]
    ) -> Optional[float]:
        """Calculate service uptime."""
        if not process_info or not process_info.get("create_time"):
            return None

        return time.time() - process_info["create_time"]



"""Command Line Interface."""

import os
import platform
import subprocess
import sys
import time

import click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from . import __version__
from .daemon import AutostartxDaemon
from .interactive import confirm_action, select_service
from .models import ServiceStatus
from .monitor import AutoRestartManager
from .service_manager import ServiceManager

console = Console()


def _get_service_identifier(manager, id, name, filter_status=None, prompt="Please select service"):
    """Helper function to get service identifier with interactive selection."""
    service_identifier = id or name
    if not service_identifier:
        services = manager.list_services()
        if filter_status:
            services = [s for s in services if s.status == filter_status]
        service = select_service(services, prompt)
        if not service:
            return None
        service_identifier = service.id
    return service_identifier


def check_systemd_support():
    """Check if the system supports systemd."""
    if platform.system() != "Linux":
        console.print("[red]❌ This tool only supports Linux systems with systemd[/red]")
        console.print(f"[red]Current system: {platform.system()}[/red]")
        sys.exit(1)
    
    try:
        # Check if systemd is available
        result = subprocess.run(
            ["systemctl", "--version"], 
            capture_output=True, 
            text=True,
            timeout=5
        )
        if result.returncode != 0:
            raise subprocess.CalledProcessError(result.returncode, "systemctl")
    except (subprocess.CalledProcessError, subprocess.TimeoutExpired, FileNotFoundError):
        console.print("[red]❌ systemd is not available on this system[/red]")
        console.print("[red]This tool requires systemd to manage services[/red]")
        sys.exit(1)


@click.group()
@click.version_option(version=__version__)
@click.option("--config", help="Configuration file path")
@click.pass_context
def cli(ctx, config):
    """Autostartx - Command-line program service management tool."""
    ctx.ensure_object(dict)
    ctx.obj["config_path"] = config


@cli.command()
@click.argument("command")
@click.option("--name", help="Service name")
@click.option("--no-auto-restart", is_flag=True, help="Disable auto restart")
@click.option("--working-dir", help="Working directory")
@click.pass_context
def add(ctx, command, name, no_auto_restart, working_dir):
    """Add new service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    # If no name specified, generate one
    if not name:
        name = f"service-{int(time.time())}"

    auto_restart = not no_auto_restart
    working_dir = working_dir or os.getcwd()

    try:
        service = manager.add_service(
            name=name,
            command=command,
            auto_restart=auto_restart,
            working_dir=working_dir,
        )

        console.print(f"✅ Service added: {service.name} ({service.id})")
        console.print(f"Command: {service.command}")
        console.print(f"Auto restart: {'Enabled' if service.auto_restart else 'Disabled'}")

        # Ask if start immediately
        try:
            if click.confirm("Start service now?", default=True):
                if manager.start_service(service.id):
                    console.print("🚀 Service started")
                else:
                    console.print("❌ Service failed to start", style="red")
        except click.Abort:
            console.print("Skipped starting service")

    except ValueError as e:
        console.print(f"❌ Error: {e}", style="red")
        sys.exit(1)
    except Exception as e:
        console.print(f"❌ Failed to add service: {e}", style="red")
        sys.exit(1)


@cli.command()
@click.option("--status", is_flag=True, help="Show detailed status")
@click.pass_context
def list(ctx, status):
    """Show service list."""
    manager = ServiceManager(ctx.obj.get("config_path"))
    services = manager.list_services()

    if not services:
        console.print("No services found")
        return

    table = Table(title="Service List")
    table.add_column("ID", style="cyan")
    table.add_column("Name", style="magenta")
    table.add_column("Status", justify="center")
    table.add_column("Command", style="blue")

    if status:
        table.add_column("PID", justify="right")
        table.add_column("Restart Count", justify="right")
        table.add_column("Created", style="dim")

    for service in services:
        status_style = _get_status_style(service.status)
        status_text = Text(service.status.value, style=status_style)

        row = [
            service.id[:8],
            service.name,
            status_text,
            (service.command[:50] + "..." if len(service.command) > 50 else service.command),
        ]

        if status:
            row.extend(
                [
                    str(service.pid) if service.pid else "-",
                    str(service.restart_count),
                    time.strftime("%Y-%m-%d %H:%M", time.localtime(service.created_at)),
                ]
            )

        table.add_row(*row)

    console.print(table)


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.pass_context
def status(ctx, id, name):
    """Show service status."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, prompt="Please select service to view status"
    )
    if not service_identifier:
        return

    status_info = manager.get_service_status(service_identifier)
    if not status_info:
        console.print("❌ Service not found", style="red")
        return

    service = status_info["service"]
    process_info = status_info["process"]
    uptime = status_info["uptime"]

    # Create status panel
    status_text = []
    status_text.append(f"ID: {service.id}")
    status_text.append(f"Name: {service.name}")
    status_text.append(f"Command: {service.command}")
    status_text.append(f"Status: {service.status.value}")
    status_text.append(f"Auto restart: {'Enabled' if service.auto_restart else 'Disabled'}")
    status_text.append(f"Restart count: {service.restart_count}")
    status_text.append(f"Working directory: {service.working_dir}")

    if process_info:
        status_text.append(f"Process ID: {process_info['pid']}")
        status_text.append(f"CPU usage: {process_info['cpu_percent']:.1f}%")

        mem_mb = process_info["memory"]["rss"] / 1024 / 1024
        status_text.append(f"Memory usage: {mem_mb:.1f} MB")

        if uptime:
            hours, remainder = divmod(int(uptime), 3600)
            minutes, seconds = divmod(remainder, 60)
            status_text.append(f"Uptime: {hours:02d}:{minutes:02d}:{seconds:02d}")

    status_text.append(
        f"Created: {time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(service.created_at))}"
    )
    status_text.append(
        f"Updated: {time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(service.updated_at))}"
    )

    panel = Panel(
        "\n".join(status_text),
        title=f"Service Status - {service.name}",
        border_style=_get_status_style(service.status),
    )
    console.print(panel)


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.pass_context
def start(ctx, id, name):
    """Start service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, filter_status=ServiceStatus.STOPPED, prompt="Please select service to start"
    )
    if not service_identifier:
        return

    if manager.start_service(service_identifier):
        console.print("🚀 Service started")
    else:
        console.print("❌ Service failed to start", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.option("--force", is_flag=True, help="Force stop")
@click.pass_context
def stop(ctx, id, name, force):
    """Stop service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, filter_status=ServiceStatus.RUNNING, prompt="Please select service to stop"
    )
    if not service_identifier:
        return

    if manager.stop_service(service_identifier, force):
        console.print("⏹️ Service stopped")
    else:
        console.print("❌ Service failed to stop", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.option("--force", is_flag=True, help="Force restart")
@click.pass_context
def restart(ctx, id, name, force):
    """Restart service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, prompt="Please select service to restart"
    )
    if not service_identifier:
        return

    if manager.restart_service(service_identifier, force):
        console.print("🔄 Service restarted")
    else:
        console.print("❌ Service failed to restart", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.pass_context
def pause(ctx, id, name):
    """Pause service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, filter_status=ServiceStatus.RUNNING, prompt="Please select service to pause"
    )
    if not service_identifier:
        return

    if manager.pause_service(service_identifier):
        console.print("⏸️ Service paused")
    else:
        console.print("❌ Service failed to pause", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.pass_context
def resume(ctx, id, name):
    """Resume service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, filter_status=ServiceStatus.PAUSED, prompt="Please select service to resume"
    )
    if not service_identifier:
        return

    if manager.resume_service(service_identifier):
        console.print("▶️ Service resumed")
    else:
        console.print("❌ Service failed to resume", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.option("--force", is_flag=True, help="Force remove")
@click.pass_context
def remove(ctx, id, name, force):
    """Remove service."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, prompt="Please select service to remove"
    )
    if not service_identifier:
        return

    service = manager.get_service(service_identifier)
    if not service:
        console.print("❌ Service not found", style="red")
        return

    # Check if service is running
    if service.status == ServiceStatus.RUNNING:
        if not force:
            console.print(f"⚠️  Service '{service.name}' is currently running", style="yellow")
            console.print("You need to stop it first before removing:")
            console.print(f"  autostartx stop {service.name}", style="cyan")
            console.print("Or use --force to stop and remove:", style="dim")
            console.print(f"  autostartx remove {service.name} --force", style="dim")
            return
        else:
            console.print(f"🛑 Stopping running service '{service.name}' before removal...")

    # Confirm removal
    if not force and not confirm_action("remove", service.name):
        console.print("Removal cancelled")
        return

    if manager.remove_service(service_identifier, force):
        console.print(f"🗑️ Service '{service.name}' removed")
    else:
        console.print("❌ Failed to remove service", style="red")


@cli.command()
@click.option("--id", help="Service ID")
@click.option("--name", help="Service name")
@click.option("--follow", "-f", is_flag=True, help="Follow log output")
@click.option("--tail", default=100, help="Show last N lines of log")
@click.option("--clear", is_flag=True, help="Clear logs")
@click.pass_context
def logs(ctx, id, name, follow, tail, clear):
    """View service logs."""
    manager = ServiceManager(ctx.obj.get("config_path"))

    service_identifier = _get_service_identifier(
        manager, id, name, prompt="Please select service to view logs"
    )
    if not service_identifier:
        return

    service = manager.get_service(service_identifier)
    if not service:
        console.print("❌ Service not found", style="red")
        return

    if clear:
        if manager.clear_service_logs(service_identifier):
            console.print("🧹 Logs cleared")
        else:
            console.print("❌ Failed to clear logs", style="red")
        return

    log_lines = manager.get_service_logs(service_identifier, tail)
    if log_lines is None:
        console.print("❌ Unable to read logs", style="red")
        return

    if not log_lines:
        console.print("📝 No logs available")
        return

    # Display historical logs
    for line in log_lines:
        console.print(line.rstrip())

    # Real-time follow mode
    if follow:
        console.print("\n--- Live logs (Ctrl+C to exit) ---")
        try:
            log_path = manager.config_manager.get_service_log_path(service.id)
            with open(log_path, encoding="utf-8") as f:
                # Move to end of file
                f.seek(0, 2)

                while True:
                    line = f.readline()
                    if line:
                        console.print(line.rstrip())
                    else:
                        time.sleep(0.1)
        except KeyboardInterrupt:
            console.print("\nLog following stopped")
        except Exception as e:
            console.print(f"❌ Log following failed: {e}", style="red")


@cli.command()
@click.option(
    "--action",
    type=click.Choice(["start", "stop", "restart", "status"]),
    default="status",
    help="Daemon operation",
)
@click.pass_context
def daemon(ctx, action):
    """Manage autostartx daemon."""
    daemon = AutostartxDaemon(ctx.obj.get("config_path"))

    if action == "start":
        console.print("🚀 Starting autostartx daemon...")
        daemon.start()
    elif action == "stop":
        console.print("🛑 Stopping autostartx daemon...")
        daemon.stop()
    elif action == "restart":
        console.print("🔄 Restarting autostartx daemon...")
        daemon.restart()
    elif action == "status":
        daemon.status()


@cli.command()
@click.pass_context
def monitor(ctx):
    """Start monitoring mode (foreground)."""
    console.print("🔍 Starting Autostartx monitoring mode...")
    console.print("Press Ctrl+C to stop monitoring")

    try:
        manager = AutoRestartManager(ctx.obj.get("config_path"))
        manager.start()
    except KeyboardInterrupt:
        console.print("\nMonitoring stopped")


@cli.command()
@click.option("--enable-autostart", is_flag=True, help="Enable system autostart after installation")
@click.pass_context
def install(ctx, enable_autostart):
    """Install autostartx to system."""
    import shutil

    # Get the script path
    script_path = sys.argv[0]

    # Determine install location
    if os.access("/usr/local/bin", os.W_OK):
        install_dir = "/usr/local/bin"
    elif os.path.expanduser("~/.local/bin"):
        install_dir = os.path.expanduser("~/.local/bin")
        os.makedirs(install_dir, exist_ok=True)
    else:
        console.print("[red]Error: No writable install directory found[/red]")
        return

    install_path = os.path.join(install_dir, "autostartx")

    try:
        shutil.copy2(script_path, install_path)
        os.chmod(install_path, 0o755)
        console.print(f"[green]Successfully installed autostartx to {install_path}[/green]")

        # Create asx alias (independent script)
        asx_path = os.path.join(install_dir, "asx")
        try:
            if os.path.exists(asx_path):
                os.remove(asx_path)

            # Create a proper asx script with correct shebang
            asx_content = """#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os

# Add the autostartx installation directory to Python path if needed
autostartx_dir = os.path.dirname(os.path.abspath(__file__))
if autostartx_dir not in sys.path:
    sys.path.insert(0, autostartx_dir)

# Import and run autostartx main function
try:
    from autostartx.cli import main
    main()
except ImportError:
    # Fallback: try to run autostartx directly
    import subprocess
    autostartx_path = os.path.join(autostartx_dir, "autostartx")
    if os.path.exists(autostartx_path):
        os.execv(sys.executable, [sys.executable, autostartx_path] + sys.argv[1:])
    else:
        print("Error: Could not find autostartx installation")
        sys.exit(1)
"""

            with open(asx_path, 'w') as f:
                f.write(asx_content)
            os.chmod(asx_path, 0o755)
            console.print(f"[green]Created asx script: {asx_path}[/green]")
        except Exception as e:
            console.print(f"[yellow]Warning: Could not create asx script: {e}[/yellow]")

        # Ask about autostart if not explicitly specified
        if not enable_autostart:
            try:
                enable_autostart = click.confirm(
                    "Do you want to enable system autostart on Linux? "
                    "(autostartx will start automatically after reboot/login)",
                    default=True
                )
            except click.Abort:
                enable_autostart = False

        # Setup autostart if requested
        if enable_autostart:
            # Check systemd support before setting up autostart
            check_systemd_support()
            console.print("\n🚀 Setting up system autostart...")
            # Use the new autostart command with the installed path
            try:
                # Update PATH to include install directory for the autostart command
                env = os.environ.copy()
                if install_dir not in env.get('PATH', ''):
                    env['PATH'] = f"{install_dir}:{env.get('PATH', '')}"

                result = subprocess.run(
                    [install_path, "autostart", "enable"],
                    env=env,
                    capture_output=True,
                    text=True
                )

                if result.returncode == 0:
                    console.print("[green]✅ Autostart enabled successfully![/green]")
                    console.print("[dim]Autostartx will start automatically after reboot[/dim]")
                else:
                    console.print(
                        f"[yellow]Warning: Could not enable autostart: {result.stderr}[/yellow]"
                    )
                    console.print(
                        f"[dim]You can enable it later with: {install_path} autostart enable[/dim]"
                    )

            except Exception as e:
                console.print(f"[yellow]Warning: Could not enable autostart: {e}[/yellow]")
                console.print(
                    f"[dim]You can enable it later with: {install_path} autostart enable[/dim]"
                )

        # Show next steps
        console.print("\n[bold green]Installation complete![/bold green]")
        console.print("Next steps:")
        console.print("1. Add services: [cyan]autostartx add \"your-command\"[/cyan]")
        if not enable_autostart:
            console.print("2. Enable autostart: [cyan]autostartx autostart enable[/cyan]")
        console.print("3. Start daemon: [cyan]autostartx daemon --action start[/cyan]")

    except Exception as e:
        console.print(f"[red]Installation failed: {e}[/red]")


@cli.command()
@click.option(
    "--action",
    type=click.Choice(["enable", "disable", "status"]),
    default="status",
    help="Autostart operation",
)
@click.pass_context
def autostart(ctx, action):
    """Manage system autostart for autostartx daemon."""
    # Check systemd support only when using autostart
    check_systemd_support()
    
    from pathlib import Path

    def get_systemd_service_path():
        return Path.home() / ".config" / "systemd" / "user" / "autostartx.service"

    def create_systemd_service():
        """Create systemd user service file."""
        service_path = get_systemd_service_path()
        service_path.parent.mkdir(parents=True, exist_ok=True)

        # Get autostartx executable path
        try:
            # Try autostartx first, then asx as fallback
            try:
                autostartx_path = subprocess.check_output(
                    ["which", "autostartx"], text=True
                ).strip()
            except subprocess.CalledProcessError:
                autostartx_path = subprocess.check_output(["which", "asx"], text=True).strip()
        except subprocess.CalledProcessError:
            # Fallback to common paths
            common_paths = [
                "/usr/local/bin/autostartx", "/usr/local/bin/asx",
                os.path.expanduser("~/.local/bin/autostartx"),
                os.path.expanduser("~/.local/bin/asx")
            ]
            for path in common_paths:
                if os.path.exists(path):
                    autostartx_path = path
                    break
            else:
                console.print("[red]Error: autostartx executable not found in PATH[/red]")
                return False

        service_content = f"""[Unit]
Description=Autostartx Service Manager
After=default.target

[Service]
Type=forking
ExecStart={autostartx_path} daemon --action start
ExecStop={autostartx_path} daemon --action stop
Restart=always
RestartSec=5
Environment=PATH={os.environ.get('PATH', '')}

[Install]
WantedBy=default.target
"""

        try:
            with open(service_path, 'w') as f:
                f.write(service_content)
            console.print(f"[green]Created systemd service: {service_path}[/green]")
            return True
        except Exception as e:
            console.print(f"[red]Failed to create systemd service: {e}[/red]")
            return False

    def enable_systemd_autostart():
        """Enable systemd user service."""
        try:
            subprocess.run(["systemctl", "--user", "daemon-reload"], check=True, capture_output=True)
            subprocess.run(["systemctl", "--user", "enable", "autostartx.service"], check=True, capture_output=True)
            console.print("[green]✅ Autostart enabled via systemd[/green]")
            return True
        except subprocess.CalledProcessError as e:
            console.print(f"[red]Failed to enable systemd service: {e}[/red]")
            return False

    def disable_systemd_autostart():
        """Disable systemd user service."""
        try:
            subprocess.run(["systemctl", "--user", "disable", "autostartx.service"], check=True, capture_output=True)
            console.print("[green]🛑 Autostart disabled[/green]")
            return True
        except subprocess.CalledProcessError as e:
            console.print(f"[red]Failed to disable systemd service: {e}[/red]")
            return False

    def check_systemd_status():
        """Check systemd service status."""
        service_path = get_systemd_service_path()
        if not service_path.exists():
            return False, "Service file not found"

        try:
            result = subprocess.run(
                ["systemctl", "--user", "is-enabled", "autostartx.service"],
                capture_output=True, text=True
            )
            if result.returncode == 0 and result.stdout.strip() == "enabled":
                return True, "enabled"
            else:
                return False, "disabled"
        except subprocess.CalledProcessError:
            return False, "unknown"

    # Linux systemd autostart implementation

    if action == "enable":
        console.print("🚀 Enabling autostartx autostart...")
        if create_systemd_service() and enable_systemd_autostart():
            console.print("[green]✅ Autostart successfully enabled![/green]")
            console.print("[dim]Autostartx daemon will start automatically after reboot[/dim]")

            # Mark all existing services with auto_restart=True as auto_start=True
            manager = ServiceManager(ctx.obj.get("config_path"))
            services = manager.list_services()
            auto_start_count = 0

            for service in services:
                if service.auto_restart and not getattr(service, 'auto_start', False):
                    service.auto_start = True
                    manager.storage.update_service(service)
                    auto_start_count += 1

            if auto_start_count > 0:
                console.print(
                    f"[green]✅ Marked {auto_start_count} service(s) for auto-start after reboot[/green]"
                )
            else:
                console.print("[dim]No services need auto-start marking[/dim]")
        else:
            console.print("[red]❌ Failed to enable autostart[/red]")

    elif action == "disable":
        console.print("🛑 Disabling autostartx autostart...")
        if disable_systemd_autostart():
            # Optionally remove service file
            service_path = get_systemd_service_path()
            if service_path.exists():
                try:
                    service_path.unlink()
                    console.print("[dim]Removed systemd service file[/dim]")
                except Exception as e:
                    console.print(f"[yellow]Warning: Could not remove service file: {e}[/yellow]")

            # Clear auto_start flag from all services
            manager = ServiceManager(ctx.obj.get("config_path"))
            services = manager.list_services()
            auto_start_count = 0

            for service in services:
                if getattr(service, 'auto_start', False):
                    service.auto_start = False
                    manager.storage.update_service(service)
                    auto_start_count += 1

            if auto_start_count > 0:
                console.print(
                    f"[green]✅ Cleared auto-start flag from {auto_start_count} service(s)[/green]"
                )
        else:
            console.print("[red]❌ Failed to disable autostart[/red]")

    elif action == "status":
        console.print("📊 Checking autostart status...")
        enabled, status = check_systemd_status()

        if enabled:
            console.print("[green]✅ Autostart is enabled[/green]")
        else:
            console.print(f"[yellow]❌ Autostart is disabled ({status})[/yellow]")

        # Check if daemon is currently running
        daemon = AutostartxDaemon(ctx.obj.get("config_path"))
        try:
            with open(daemon.pidfile) as pf:
                pid = int(pf.read().strip())
            try:
                os.kill(pid, 0)
                console.print("[green]🟢 Daemon is currently running[/green]")
            except OSError:
                console.print("[yellow]🟡 Daemon is not running[/yellow]")
        except (OSError, ValueError):
            console.print("[yellow]🟡 Daemon is not running[/yellow]")

@cli.command()
@click.option("--remove-config", is_flag=True, help="Also remove configuration and data files")
@click.pass_context
def uninstall(ctx, remove_config):
    """Uninstall autostartx from system."""
    from pathlib import Path

    console.print("🗑️ Uninstalling autostartx...")

    # First, disable autostart
    try:
        result = subprocess.run(
            ["autostartx", "autostart", "disable"],
            capture_output=True,
            text=True
        )
        if result.returncode == 0:
            console.print("[green]✅ Autostart disabled[/green]")
    except Exception as e:
        console.print(f"[yellow]Warning: Could not disable autostart: {e}[/yellow]")

    # Stop daemon if running
    try:
        daemon = AutostartxDaemon(ctx.obj.get("config_path"))
        daemon.stop()
        console.print("[green]✅ Daemon stopped[/green]")
    except Exception as e:
        console.print(f"[yellow]Warning: Could not stop daemon: {e}[/yellow]")

    # Remove executable and alias
    removed_paths = []
    for path in ["/usr/local/bin/autostartx", os.path.expanduser("~/.local/bin/autostartx")]:
        if os.path.exists(path):
            try:
                os.remove(path)
                removed_paths.append(path)
                console.print(f"[green]✅ Removed: {path}[/green]")
            except Exception as e:
                console.print(f"[red]❌ Failed to remove {path}: {e}[/red]")

    # Remove asx aliases
    for path in ["/usr/local/bin/asx", os.path.expanduser("~/.local/bin/asx")]:
        if os.path.exists(path):
            try:
                os.remove(path)
                console.print(f"[green]✅ Removed alias: {path}[/green]")
            except Exception as e:
                console.print(f"[red]❌ Failed to remove alias {path}: {e}[/red]")

    if not removed_paths:
        console.print("[yellow]⚠️ No autostartx executable found to remove[/yellow]")

    # Optionally remove config and data
    if remove_config:
        config_dirs = [
            Path.home() / ".config" / "autostartx",
            Path.home() / ".local" / "share" / "autostartx"
        ]

        for config_dir in config_dirs:
            if config_dir.exists():
                try:
                    import shutil
                    shutil.rmtree(config_dir)
                    console.print(f"[green]✅ Removed: {config_dir}[/green]")
                except Exception as e:
                    console.print(f"[red]❌ Failed to remove {config_dir}: {e}[/red]")
    else:
        console.print("[dim]Configuration and data files preserved[/dim]")
        console.print("[dim]Use --remove-config to remove all data[/dim]")

    console.print("\n[bold green]Uninstallation complete![/bold green]")


def _get_status_style(status: ServiceStatus) -> str:
    """Get status style."""
    styles = {
        ServiceStatus.RUNNING: "green",
        ServiceStatus.STOPPED: "red",
        ServiceStatus.PAUSED: "yellow",
        ServiceStatus.FAILED: "bright_red",
        ServiceStatus.STARTING: "cyan",
    }
    return styles.get(status, "white")


def main():
    """Main entry function."""
    try:
        cli()
    except KeyboardInterrupt:
        console.print("\nOperation cancelled")
        sys.exit(0)
    except Exception as e:
        console.print(f"❌ Error occurred: {e}", style="red")
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Process management module."""

import os
import signal
import subprocess
import time
from typing import Any, Dict, List, Optional

import psutil

from .config import ConfigManager
from .models import ServiceInfo, ServiceStatus


class ProcessInfo:
    """Process information class."""

    def __init__(self, pid: int):
        self.pid = pid
        self._process = psutil.Process(pid) if psutil.pid_exists(pid) else None

    @property
    def exists(self) -> bool:
        """Whether process exists."""
        return self._process is not None and self._process.is_running()

    @property
    def status(self) -> str:
        """Process status."""
        if not self.exists:
            return "not_found"
        try:
            return self._process.status()
        except psutil.NoSuchProcess:
            return "not_found"

    @property
    def memory_info(self) -> Dict[str, int]:
        """Memory usage information."""
        if not self.exists:
            return {"rss": 0, "vms": 0}
        try:
            mem = self._process.memory_info()
            return {"rss": mem.rss, "vms": mem.vms}
        except psutil.NoSuchProcess:
            return {"rss": 0, "vms": 0}

    @property
    def cpu_percent(self) -> float:
        """CPU usage rate."""
        if not self.exists:
            return 0.0
        try:
            return self._process.cpu_percent()
        except psutil.NoSuchProcess:
            return 0.0

    @property
    def create_time(self) -> float:
        """Process creation time."""
        if not self.exists:
            return 0.0
        try:
            return self._process.create_time()
        except psutil.NoSuchProcess:
            return 0.0

    def terminate(self) -> bool:
        """Gracefully terminate process."""
        if not self.exists:
            return True

        try:
            self._process.terminate()
            # Wait for process to end
            self._process.wait(timeout=10)
            return True
        except (psutil.NoSuchProcess, psutil.TimeoutExpired):
            return self.kill()

    def kill(self) -> bool:
        """Force kill process."""
        if not self.exists:
            return True

        try:
            self._process.kill()
            self._process.wait(timeout=5)
            return True
        except (psutil.NoSuchProcess, psutil.TimeoutExpired):
            # Process may have ended or cannot be killed
            return not self.exists


class ProcessManager:
    """Process manager."""

    def __init__(self, config_manager: ConfigManager):
        self.config_manager = config_manager

    def start_service(self, service: ServiceInfo) -> bool:
        """Start service."""
        if service.pid and self.is_process_running(service.pid):
            return False  # Process already running

        try:
            # Get log file path
            log_path = self.config_manager.get_service_log_path(service.id)
            os.makedirs(os.path.dirname(log_path), exist_ok=True)

            # Start process
            with open(log_path, "a", encoding="utf-8") as log_file:
                # Write startup log
                log_file.write(f"\n=== Service started: {time.strftime('%Y-%m-%d %H:%M:%S')} ===\n")
                log_file.flush()

                # Parse command
                cmd_parts = self._parse_command(service.command)

                # Set environment variables
                env = os.environ.copy()
                env.update(service.env_vars)

                # Start process
                process = subprocess.Popen(
                    cmd_parts,
                    stdout=log_file,
                    stderr=subprocess.STDOUT,
                    cwd=service.working_dir or os.getcwd(),
                    env=env,
                    start_new_session=True,  # Create new process group
                )

                # Store the process PID
                service.pid = process.pid
                print(f"[DEBUG] Started service {service.name} with PID {process.pid}")
                
                # For sudo commands, try to find the actual child process
                if cmd_parts[0] == "sudo":
                    time.sleep(0.5)  # Give time for child process to start
                    try:
                        parent_process = psutil.Process(process.pid)
                        children = parent_process.children(recursive=True)
                        if children:
                            # Use the first child process as the actual service PID
                            actual_pid = children[0].pid
                            service.pid = actual_pid
                            print(f"[DEBUG] Found child process PID {actual_pid} for sudo command")
                    except (psutil.NoSuchProcess, psutil.AccessDenied):
                        # If we can't get child processes, use original PID
                        pass
                
                service.update_status(ServiceStatus.RUNNING)
                return True

        except Exception as e:
            print(f"Failed to start service: {e}")
            service.update_status(ServiceStatus.FAILED)
            return False

    def stop_service(self, service: ServiceInfo, force: bool = False) -> bool:
        """Stop service."""
        if not service.pid:
            service.update_status(ServiceStatus.STOPPED)
            return True

        process_info = ProcessInfo(service.pid)
        if not process_info.exists:
            service.pid = None
            service.update_status(ServiceStatus.STOPPED)
            return True

        # Stop process
        success = process_info.kill() if force else process_info.terminate()

        if success:
            service.pid = None
            service.update_status(ServiceStatus.STOPPED)

            # Write stop log
            log_path = self.config_manager.get_service_log_path(service.id)
            try:
                with open(log_path, "a", encoding="utf-8") as log_file:
                    log_file.write(
                        f"\n=== Service stopped: {time.strftime('%Y-%m-%d %H:%M:%S')} ===\n"
                    )
            except Exception:
                pass  # Ignore log write errors

        return success

    def restart_service(self, service: ServiceInfo, force: bool = False) -> bool:
        """Restart service."""
        # First stop
        if not self.stop_service(service, force):
            return False

        # Wait a moment
        time.sleep(1)

        # Then start
        return self.start_service(service)

    def pause_service(self, service: ServiceInfo) -> bool:
        """Pause service."""
        if not service.pid:
            return False

        process_info = ProcessInfo(service.pid)
        if not process_info.exists:
            service.pid = None
            service.update_status(ServiceStatus.STOPPED)
            return False

        try:
            os.kill(service.pid, signal.SIGSTOP)
            service.update_status(ServiceStatus.PAUSED)
            return True
        except (OSError, ProcessLookupError):
            return False

    def resume_service(self, service: ServiceInfo) -> bool:
        """Resume service."""
        if not service.pid:
            return False

        process_info = ProcessInfo(service.pid)
        if not process_info.exists:
            service.pid = None
            service.update_status(ServiceStatus.STOPPED)
            return False

        try:
            os.kill(service.pid, signal.SIGCONT)
            service.update_status(ServiceStatus.RUNNING)
            return True
        except (OSError, ProcessLookupError):
            return False

    def get_process_info(self, service: ServiceInfo) -> Optional[Dict[str, Any]]:
        """Get process information."""
        if not service.pid:
            return None

        process_info = ProcessInfo(service.pid)
        if not process_info.exists:
            return None

        return {
            "pid": service.pid,
            "status": process_info.status,
            "cpu_percent": process_info.cpu_percent,
            "memory": process_info.memory_info,
            "create_time": process_info.create_time,
        }

    def is_process_running(self, pid: int) -> bool:
        """Check if process is running."""
        try:
            if not psutil.pid_exists(pid):
                print(f"[DEBUG] Process {pid} not found in system")
                return False
                
            process = psutil.Process(pid)
            
            # Check if process is running
            if not process.is_running():
                print(f"[DEBUG] Process {pid} exists but not running (status: {process.status()})")
                return False
                
            print(f"[DEBUG] Process {pid} check passed - running normally")
            return True
            
        except psutil.NoSuchProcess:
            print(f"[DEBUG] Process {pid} no longer exists")
            return False
        except psutil.AccessDenied:
            print(f"[DEBUG] Access denied for process {pid} - assuming it's running")
            # If we can't access it, assume it's running to avoid false restarts
            return True
        except Exception as e:
            print(f"[DEBUG] Process {pid} check error: {e} - assuming it's running")
            # If we can't determine status, assume it's running to avoid false restarts
            return True

    def _parse_command(self, command: str) -> List[str]:
        """Parse command string."""
        # Simple command parsing, supports quotes
        import shlex

        return shlex.split(command)



"""Service data storage management."""

import json
import os
import uuid
from typing import Dict, List, Optional

from .config import ConfigManager
from .models import ServiceInfo, ServiceStatus


class ServiceStorage:
    """Service data storage manager."""

    def __init__(self, config_manager: ConfigManager):
        self.config_manager = config_manager
        self.db_path = config_manager.get_services_db_path()
        self._services: Dict[str, ServiceInfo] = {}
        self.load_services()

    def load_services(self) -> None:
        """Load service data from file."""
        if not os.path.exists(self.db_path):
            self._services = {}
            return

        try:
            with open(self.db_path, encoding="utf-8") as f:
                data = json.load(f)

            self._services = {}
            for service_id, service_data in data.items():
                try:
                    service = ServiceInfo.from_dict(service_data)
                    self._services[service_id] = service
                except Exception as e:
                    print(f"Warning: Failed to load service {service_id}: {e}")

        except Exception as e:
            print(f"Warning: Failed to load service data: {e}")
            self._services = {}

    def save_services(self) -> None:
        """Save service data to file."""
        try:
            # Ensure directory exists
            os.makedirs(os.path.dirname(self.db_path), exist_ok=True)

            data = {}
            for service_id, service in self._services.items():
                data[service_id] = service.to_dict()

            with open(self.db_path, "w", encoding="utf-8") as f:
                json.dump(data, f, indent=2, ensure_ascii=False)

        except Exception as e:
            print(f"Error: Failed to save service data: {e}")

    def add_service(
        self,
        name: str,
        command: str,
        auto_restart: bool = True,
        working_dir: str = "",
        env_vars: Optional[Dict[str, str]] = None,
    ) -> ServiceInfo:
        """Add new service."""
        service_id = self._generate_service_id()

        # Check name conflict
        if self.get_service_by_name(name):
            raise ValueError(f"Service name '{name}' already exists")

        service = ServiceInfo(
            id=service_id,
            name=name,
            command=command,
            auto_restart=auto_restart,
            working_dir=working_dir or os.getcwd(),
            env_vars=env_vars or {},
            max_restart_attempts=self.config_manager.config.max_restart_attempts,
            restart_delay=self.config_manager.config.restart_delay,
        )

        self._services[service_id] = service
        self.save_services()
        return service

    def get_service(self, service_id: str) -> Optional[ServiceInfo]:
        """Get service by ID."""
        return self._services.get(service_id)

    def get_service_by_name(self, name: str) -> Optional[ServiceInfo]:
        """Get service by name."""
        for service in self._services.values():
            if service.name == name:
                return service
        return None

    def get_all_services(self) -> List[ServiceInfo]:
        """Get all services."""
        return list(self._services.values())

    def update_service(self, service: ServiceInfo) -> None:
        """Update service information."""
        if service.id in self._services:
            self._services[service.id] = service
            self.save_services()
        else:
            raise ValueError(f"Service {service.id} does not exist")

    def remove_service(self, service_id: str) -> bool:
        """Remove service."""
        if service_id in self._services:
            del self._services[service_id]
            self.save_services()
            return True
        return False

    def find_service(self, service_id_or_name: str) -> Optional[ServiceInfo]:
        """Find service by ID or name."""
        # First try to find by ID
        service = self.get_service(service_id_or_name)
        if service:
            return service

        # Then try to find by name
        return self.get_service_by_name(service_id_or_name)

    def get_services_by_status(self, status: ServiceStatus) -> List[ServiceInfo]:
        """Get service list by status."""
        return [service for service in self._services.values() if service.status == status]

    def _generate_service_id(self) -> str:
        """Generate unique service ID."""
        while True:
            service_id = str(uuid.uuid4())[:8]
            if service_id not in self._services:
                return service_id
